@@ -2,18 +2,18 @@
 #' @param git_pkg A list of three elements: "package_name", the name of the
 #'   package, "repo_url", the repository's url, "commit", the commit hash of
 #'   interest.
+#' @param ... Further arguments passed down to methods.
 #' @return A character. The Nix definition to download and build the R package
 #'   from GitHub.
 #' @noRd
-fetchgit <- function(git_pkg) {
+fetchgit <- function(git_pkg, ...) {
   package_name <- git_pkg$package_name
   repo_url <- git_pkg$repo_url
   commit <- git_pkg$commit
-
-  output <- get_sri_hash_deps(repo_url, commit)
+  output <- nix_hash(repo_url, commit, ...)
   sri_hash <- output$sri_hash
-  # If package has no remote dependencies
 
+  # If package has no remote dependencies
   imports <- output$deps$imports
   imports <- paste(c("", imports), collapse = "\n          ")
 
@@ -32,18 +32,13 @@ fetchgit <- function(git_pkg) {
     # if no remote dependencies
 
     output <- main_package_expression
-  } else { # if there are remote dependencies, start over
-    # don't include remote dependencies twice
-    # this can happen if a remote dependency of a remote dependency
-    # is already present as a remote dependency
-    remotes_remotes <- unique(unlist(lapply(remotes, get_remote)))
-    remotes <- remotes[!sapply(remotes, function(pkg) {
-      pkg$package_name %in% remotes_remotes
-    })]
+  } else {
+    # if there are remote dependencies, start over
 
-    remote_packages_expressions <- fetchgits(remotes)
+    remote_packages_expressions <- fetchgits(remotes, ...)
 
-    output <- paste0(remote_packages_expressions,
+    output <- paste0(
+      remote_packages_expressions,
       main_package_expression,
       collapse = "\n"
     )
@@ -62,12 +57,14 @@ fetchgit <- function(git_pkg) {
 #' @return A character. Part of the Nix definition to download and build the R package
 #' from the CRAN archives.
 #' @noRd
-generate_git_nix_expression <- function(package_name,
-                                        repo_url,
-                                        commit,
-                                        sri_hash,
-                                        imports,
-                                        remotes = NULL) {
+generate_git_nix_expression <- function(
+  package_name,
+  repo_url,
+  commit,
+  sri_hash,
+  imports,
+  remotes = NULL
+) {
   # If there are remote dependencies, pass this string
   flag_remote_deps <- if (is.list(remotes) && length(remotes) == 0) {
     ""
@@ -112,7 +109,8 @@ fetchzip <- function(archive_pkg, sri_hash = NULL) {
 
   cran_archive_link <- paste0(
     "https://cran.r-project.org/src/contrib/Archive/",
-    pkgs[1], "/",
+    pkgs[1],
+    "/",
     paste0(pkgs[1], "_", pkgs[2]),
     ".tar.gz"
   )
@@ -121,7 +119,7 @@ fetchzip <- function(archive_pkg, sri_hash = NULL) {
   repo_url <- cran_archive_link
 
   if (is.null(sri_hash)) {
-    output <- get_sri_hash_deps(repo_url, commit = NULL)
+    output <- nix_hash(repo_url, commit = NULL)
     sri_hash <- output$sri_hash
     imports <- output$deps$imports
     imports <- paste(c("", imports), collapse = "\n          ")
@@ -175,10 +173,11 @@ remove_base <- function(list_imports) {
 #' Finds dependencies of a package from the DESCRIPTION file
 #' @param path path to package
 #' @param commit_date date of commit
+#' @param ... Further arguments passed down to methods.
 #' @importFrom utils untar
 #' @return Atomic vector of packages
 #' @noRd
-get_imports <- function(path, commit_date) {
+get_imports <- function(path, commit_date, ...) {
   tmpdir <- tempdir()
 
   tmp_dir <- tempfile(pattern = "file", tmpdir = tmpdir, fileext = "")
@@ -199,7 +198,9 @@ get_imports <- function(path, commit_date) {
   } else if (grepl("DESCRIPTION", path)) {
     desc_path <- path
   } else {
-    stop("Path is neither a .tar.gz archive, nor pointing to a DESCRIPTION file directly.")
+    stop(
+      "Path is neither a .tar.gz archive, nor pointing to a DESCRIPTION file directly."
+    )
   }
 
   columns_of_interest <- c("Depends", "Imports", "LinkingTo")
@@ -218,14 +219,13 @@ get_imports <- function(path, commit_date) {
     remotes <- gsub("\n", "", x = unlist(strsplit(remotes$Remotes, ",")))
     # Remove PR if present because this is difficult to handle
     remotes <- sub("#.*$", "", remotes)
-    # Only keep @ part if it is a commit sha
+    # Only keep part after @ if it is a commit sha(7-40 hex chars)
     remotes <- unname(sapply(remotes, function(x) {
       parts <- strsplit(x, "@")[[1]]
       if (length(parts) == 1) {
         return(parts[1])
       }
       ref <- parts[2]
-      # Keep only if it looks like a SHA (7-40 hex chars)
       if (grepl("^[0-9a-f]{7,40}$", ref)) {
         return(x)
       }
@@ -242,12 +242,13 @@ get_imports <- function(path, commit_date) {
 
     # try to get commit hash for each package if not already provided
     remote_pkgs_refs <- lapply(remote_pkgs_names_and_refs, function(x) {
-      resolve_package_commit(x, commit_date, remotes)
+      resolve_package_commit(x, commit_date, remotes, ...)
     })
 
     urls <- paste0(
       "https://github.com/",
-      remote_pkgs_usernames, "/",
+      remote_pkgs_usernames,
+      "/",
       remote_pkgs_names
     )
 
@@ -282,7 +283,11 @@ get_imports <- function(path, commit_date) {
 
   if (length(namespace_imports) > 0) {
     # Get package names from `importFrom` statements
-    namespace_imports_pkgs <- gsub("importFrom\\(([^,]+).*", "\\1", namespace_imports)
+    namespace_imports_pkgs <- gsub(
+      "importFrom\\(([^,]+).*",
+      "\\1",
+      namespace_imports
+    )
     # Remove quotes, which is sometimes necessary
     # example: https://github.com/cran/AER/blob/master/NAMESPACE
     namespace_imports_pkgs <- gsub("[\"']", "", namespace_imports_pkgs)
@@ -367,32 +372,69 @@ fetchlocals <- function(local_r_pkgs) {
 }
 
 
-
 #' fetchgits Downloads and installs packages hosted on Git. Wraps `fetchgit()`
 #' to handle multiple packages
 #' @param git_pkgs A list of three elements: "package_name", the name of the
 #' package, "repo_url", the repository's url and "commit", the commit hash of
 #' interest. This argument can also be a list of lists of these three elements.
+#' @param ... Further arguments passed down to methods.
 #' @return A character. The Nix definition to download and build the R package
 #' from GitHub.
 #' @noRd
-fetchgits <- function(git_pkgs) {
-  if (!all(vapply(git_pkgs, is.list, logical(1)))) {
-    fetchgit(git_pkgs)
-  } else if (all(vapply(git_pkgs, is.list, logical(1)))) {
-    # Re-order list of git packages by "package name"
-    git_pkgs <- git_pkgs[order(sapply(git_pkgs, "[[", "package_name"))]
+fetchgits <- function(git_pkgs, ...) {
+  # Check if ignore_remotes_cache was passed
+  # If not passed, ignore_remotes_cache is FALSE
+  args <- list(...)
+  ignore_remotes_cache <- if (!is.null(args$ignore_remotes_cache))
+    args$ignore_remotes_cache else FALSE
 
-    paste(lapply(git_pkgs, fetchgit), collapse = "\n")
-  } else {
-    stop(
-      paste0(
-        "There is something wrong with the input. ",
-        "Make sure it is either a list of three elements ",
-        "'package_name', 'repo_url' and 'commit', or ",
-        "a list of lists with these three elements"
+  if (!ignore_remotes_cache) {
+    cache_file <- get_cache_file()
+    cache <- readRDS(cache_file)
+    if (!all(vapply(git_pkgs, is.list, logical(1)))) {
+      if (git_pkgs$package_name %in% cache$seen_packages) {
+        return("")
+      }
+      cache$seen_packages <- c(cache$seen_packages, git_pkgs$package_name)
+      saveRDS(cache, cache_file)
+      fetchgit(git_pkgs, ...)
+    } else if (all(vapply(git_pkgs, is.list, logical(1)))) {
+      # Re-order list of git packages by "package name"
+      git_pkgs <- git_pkgs[order(sapply(git_pkgs, "[[", "package_name"))]
+      # Filter out already processed packages
+      git_pkgs <- git_pkgs[
+        !sapply(
+          git_pkgs,
+          function(x) x$package_name %in% cache$seen_packages
+        )
+      ]
+
+      cache$seen_packages <- c(
+        cache$seen_packages,
+        sapply(git_pkgs, "[[", "package_name")
       )
-    )
+
+      saveRDS(cache, cache_file)
+      paste(lapply(git_pkgs, function(pkg) fetchgit(pkg, ...)), collapse = "\n")
+    } else {
+      stop(
+        "There is something wrong with the input. Make sure it is either a list of three elements ",
+        "'package_name', 'repo_url' and 'commit', or a list of lists with these three elements"
+      )
+    }
+  } else {
+    # When ignoring cache, process all packages without checking cache
+    if (!all(vapply(git_pkgs, is.list, logical(1)))) {
+      fetchgit(git_pkgs, ...)
+    } else if (all(vapply(git_pkgs, is.list, logical(1)))) {
+      git_pkgs <- git_pkgs[order(sapply(git_pkgs, "[[", "package_name"))]
+      paste(lapply(git_pkgs, fetchgit, ...), collapse = "\n")
+    } else {
+      stop(
+        "There is something wrong with the input. Make sure it is either a list of three elements ",
+        "'package_name', 'repo_url' and 'commit', or a list of lists with these three elements"
+      )
+    }
   }
 }
 
@@ -421,39 +463,28 @@ fetchzips <- function(archive_pkgs) {
 #' fetchpkgs Downloads and installs packages from CRAN archives or GitHub
 #' @param git_pkgs List of Git packages with name, url and commit
 #' @param archive_pkgs Vector of CRAN archive package names
+#' @param ... Further arguments passed down to methods.
 #' @return Nix definition string for building the packages
 #' @noRd
-fetchpkgs <- function(git_pkgs, archive_pkgs) {
-  # Only include git packages that aren't already remote dependencies
-  if (all(sapply(git_pkgs, is.list))) {
-    all_remotes <- unique(unlist(lapply(git_pkgs, get_remote)))
-    git_pkgs <- git_pkgs[!sapply(git_pkgs, function(pkg) {
-      pkg$package_name %in% all_remotes
-    })]
+fetchpkgs <- function(git_pkgs, archive_pkgs, ...) {
+  args <- list(...)
+  ignore_remotes_cache <- if (!is.null(args$ignore_remotes_cache))
+    args$ignore_remotes_cache else FALSE
+
+  # Initialize cache if git packages are present and not ignoring cache
+  if (!is.null(git_pkgs) && !ignore_remotes_cache) {
+    cache_file <- get_cache_file()
+    on.exit(unlink(cache_file))
   }
 
   # Combine git and archive package definitions
   paste(
-    fetchgits(git_pkgs),
+    fetchgits(git_pkgs, ...),
     fetchzips(archive_pkgs),
     collapse = "\n"
   )
 }
 
-#' get_remote Retrieves the names of remote dependencies for a given Git package
-#' @param git_pkg A list of three elements: "package_name", the name of the
-#'   package, "repo_url", the repository's URL, and "commit", the commit hash of
-#'   interest.
-#' @return A character vector containing the names of remote dependencies.
-#' @noRd
-get_remote <- function(git_pkg) {
-  repo_url <- git_pkg$repo_url
-  commit <- git_pkg$commit
-  output <- get_sri_hash_deps(repo_url, commit)
-  remotes <- output$deps$remotes
-  remote_package_names <- sapply(remotes, `[[`, "package_name")
-  return(remote_package_names)
-}
 
 #' get_commit_date Retrieves the date of a commit from a Git repository
 #' @param repo The GitHub repository (e.g. "r-lib/usethis")
@@ -564,7 +595,6 @@ download_all_commits <- function(repo, date) {
         n_commits <- length(commits$sha)
         if (n_commits == 0) break
 
-
         idx <- (commit_count + 1):(commit_count + n_commits)
         all_commits$sha[idx] <- commits$sha
         all_commits$date[idx] <- as.POSIXct(
@@ -610,129 +640,102 @@ get_closest_commit <- function(commits_df, target_date) {
 }
 
 #' resolve_package_commit Resolves the commit SHA for a package based on a date
-#' @param remote_pkg_name_and_ref A list containing the package name and optionally a ref
+#' @param remote_pkg_name_and_ref A list containing the package name and
+#'   optionally a ref
 #' @param date The target date to find the closest commit
-#' @param remotess A character vector of remotes
-#' @return A character. The commit SHA of the closest commit to the target date or "HEAD" if API fails
+#' @param remotes A character vector of remotes
+#' @param ... Further arguments passed down to methods.
+#' @return A character. The commit SHA of the closest commit to the target date
+#'   or "HEAD" if API fails
 #' @noRd
-resolve_package_commit <- function(remote_pkg_name_and_ref, date, remotes) {
-  # Check if remote is a list with a package name and a ref
-  if (length(remote_pkg_name_and_ref) == 2) {
-    # Keep existing ref if present
-    return(remote_pkg_name_and_ref[[2]])
+resolve_package_commit <- function(
+  remote_pkg_name_and_ref,
+  date,
+  remotes,
+  ...
+) {
+  pkg_name <- remote_pkg_name_and_ref[[1]]
+
+  # Check if ignore_remotes_cache was passed, otherwise set to FALSE
+  args <- list(...)
+  ignore_remotes_cache <- if (!is.null(args$ignore_remotes_cache))
+    args$ignore_remotes_cache else FALSE
+
+  # Check if package is already in cache
+  if (!ignore_remotes_cache) {
+    cache_file <- get_cache_file()
+    cache <- readRDS(cache_file)
+    pkg_matches <- grep(paste0("^", pkg_name, "@"), cache$commit_cache)
+
+    # Return commit from cache if found
+    if (length(pkg_matches) > 0) {
+      return(cache$commit_cache[pkg_matches[1]])
+    }
+  }
+  # Store package name and ref in cache key if ref (commit-sha) is provided
+  # otherwise set to NULL
+  # example: package_name@commit_sha, e.g., schex@031320d was earlier split
+  # into a list of two elements: `package_name` and `commit_sha, e.g., `schex`, `031320d`
+  # and is now stored in `cache_key` as `schex@031320d` for caching
+  if (!ignore_remotes_cache) {
+    cache_key <- if (length(remote_pkg_name_and_ref) == 2) {
+      paste0(pkg_name, "@", remote_pkg_name_and_ref[[2]])
+    } else {
+      NULL
+    }
+  }
+
+  # If ref (commit hash, e.g. `031320d`) is provided, use it
+  commit <- if (length(remote_pkg_name_and_ref) == 2) {
+    remote_pkg_name_and_ref[[2]]
   } else if (length(remote_pkg_name_and_ref) == 1) {
     # For packages without ref, try to find closest one by date
     # fallback to HEAD if API fails
-    result <- tryCatch(
+    tryCatch(
       {
         remotes_fetch <- remotes[grepl(remote_pkg_name_and_ref, remotes)]
         all_commits <- download_all_commits(remotes_fetch, date)
         closest_commit <- get_closest_commit(all_commits, date)
-        closest_commit$sha
+        commit <- closest_commit$sha
+        cache_key <- paste0(pkg_name, "@", commit)
+        commit
       },
       error = function(e) {
-        message(
-          paste0(
-            "Failed to get closest commit for ",
-            remotes_fetch,
-            ": ",
-            e$message,
-            ".\nFalling back to <<< HEAD >>>\n"
-          )
-        )
-        return("HEAD")
+        message(paste0(
+          "Failed to get closest commit for ",
+          remotes_fetch,
+          ": ",
+          e$message,
+          ".\nFalling back to <<< HEAD >>>\n"
+        ))
+        "HEAD"
       }
     )
-    return(result)
   } else {
     stop("remote_pkg_name_and_ref must be a list of length 1 or 2")
   }
+  # If not ignoring cache, Update cache with new cache_key (e.g. `schex@031320d`)
+  if (!ignore_remotes_cache) {
+    cache$commit_cache <- c(cache$commit_cache, cache_key)
+    saveRDS(cache, cache_file)
+  }
+
+  return(commit)
 }
 
-
-#' remove_duplicate_entries Internal function post-processes `default.nix`
-#' files. When remote packages have remote dependencies, it can happen
-#' that there are duplicated entries in the generated `default.nix` files.
-#' This function removes duplicated blocks.
-#' @param default.nix Character, default.nix lines.
+#' Get shared cache file path
+#' @return Path to shared cache file
 #' @noRd
-remove_duplicate_entries <- function(default.nix) {
-
-    # nolint start: object_name_linter
-    #default.nix_path <- file.path(default.nix_path)
-    # nolint end
-
-    lines <- default.nix
-
-    # To store output lines
-    out_lines <- character(0)
-
-    # A vector to track which package blocks have been seen
-    seen <- character(0)
-
-    # A vector to track package names for which duplicates were removed
-    removed <- character(0)
-
-    # Variables to accumulate a multi-line block
-    in_block <- FALSE
-    block_lines <- character(0)
-    block_name <- NULL
-
-    # Process each line in order
-    for (line in lines) {
-      if (!in_block) {
-        # Look for the start of a package block:
-        # A package block is assumed to start with a line like:
-        #    <name> = (<rest>
-        if (grepl("^\\s*([a-zA-Z0-9_]+)\\s*=\\s*\\(", line)) {
-          # Extract the package name from the start of the line.
-          block_name <- sub("^\\s*([a-zA-Z0-9_]+)\\s*=.*", "\\1", line)
-          in_block <- TRUE
-          block_lines <- line
-        } else {
-          # Not a block start, simply add the line to output.
-          out_lines <- c(out_lines, line)
-        }
-      } else {
-        # We are inside a block; accumulate the line.
-        block_lines <- c(block_lines, line)
-
-        # Check if the current line marks the end of the block.
-        # Here we assume a block ends with a line that has a
-        # closing parenthesis and semicolon.
-        if (grepl("\\)\\s*;\\s*$", line)) {
-          # If this package has not been seen yet, add the block to the output.
-          if (!(block_name %in% seen)) {
-            out_lines <- c(out_lines, block_lines)
-            seen <- c(seen, block_name)
-          } else {
-            if (identical(Sys.getenv("TESTTHAT"), "false")) {
-              message(sprintf("Duplicate block for '%s' already removed, skipping.",
-                              block_name))
-              removed <- c(removed, block_name)
-            }
-          }
-          # Reset block tracking variables
-          in_block <- FALSE
-          block_lines <- character(0)
-          block_name <- NULL
-        }
-      }
-    }
-
-  # Hide messages when testing
-  if (identical(Sys.getenv("TESTTHAT"), "false")) {
-      # At the end, print a message listing all removed packages (unique names)
-      removed_unique <- unique(removed)
-      if (length(removed_unique) > 0) {
-        message("Removed duplicate blocks for the following packages: ",
-                paste(removed_unique, collapse = ", "))
-      } else {
-        message("No duplicate package blocks were found.")
-      }
-    }
-
-    out_lines
-
+get_cache_file <- function() {
+  cache_file <- file.path(tempdir(), "package_cache.rds")
+  if (!file.exists(cache_file)) {
+    saveRDS(
+      list(
+        "seen_packages" = character(0),
+        "commit_cache" = character(0)
+      ),
+      cache_file
+    )
+  }
+  return(cache_file)
 }
-
