@@ -36,6 +36,10 @@
 #'   See the
 #'   `vignette("d2- installing-system-tools-and-texlive-packages-in-a-nix-environment")`
 #'   for more details.
+#' @param py_pkgs List. A list of two elements, `py_version` and `py_pkgs`.
+#'   `py_version` must be of the form `"3.12"` for Python 3.12 and `py_pkgs`
+#'   must be an atomic vector of packages names, for example
+#'   `py_pkgs = c("polars", "plotnine", "great-tables")`.
 #' @param ide Character, defaults to "none". If you wish to use RStudio to work
 #'   interactively use "rstudio" or "rserver" for the server version. Use "code"
 #'   for Visual Studio Code or "codium" for Codium, or "positron" for Positron.
@@ -196,6 +200,7 @@ rix <- function(
   git_pkgs = NULL,
   local_r_pkgs = NULL,
   tex_pkgs = NULL,
+  py_pkgs = NULL,
   ide = "none",
   project_path,
   overwrite = FALSE,
@@ -373,13 +378,20 @@ for more details."
     ""
   }
 
-  # If there are R packages local packages, passes the string "local_r_pkgs" to buildInputs
+  # If there are local R packages, passes the string "local_r_pkgs" to buildInputs
   flag_local_r_pkgs <- if (is.null(local_r_pkgs)) {
     ""
   } else {
     "local_r_pkgs"
   }
 
+  # If there are Python packages, passes the string "local_r_pkgs" to buildInputs
+  flag_py_pkgs <- if (is.null(py_pkgs)) {
+    ""
+  } else {
+    "pypkgs"
+  }
+
   # If there are wrapped packages (for example for RStudio), passes the "wrapped_pkgs"
   # to buildInputs
   flag_wrapper <- if (ide %in% names(attrib) && flag_rpkgs != "")
@@ -407,8 +419,9 @@ for more details."
       ignore_remotes_cache = ignore_remotes_cache
     ),
     generate_tex_pkgs(tex_pkgs),
+    generate_py_pkgs(py_pkgs, flag_py_pkgs),
     generate_local_r_pkgs(local_r_pkgs, flag_local_r_pkgs),
-    generate_system_pkgs(system_pkgs, r_pkgs, ide),
+    generate_system_pkgs(system_pkgs, r_pkgs, py_pkgs, ide),
     generate_wrapped_pkgs(
       ide,
       attrib,
@@ -420,10 +433,12 @@ for more details."
       flag_git_archive,
       flag_rpkgs,
       flag_tex_pkgs,
+      flag_py_pkgs,
       flag_local_r_pkgs,
       flag_wrapper,
       shell_hook
     ),
+    generate_inherit(),
     collapse = "\n"
   )
 
